# -*- coding: utf-8 -*-

import grokcore.view as grok

from dolmen.file import INamedFile
from zope.interface import Interface
from zope.component import getMultiAdapter
from zope.app.file.interfaces import IFile
from zope.security.interfaces import Unauthorized
from zope.security.management import checkPermission
from zope.publisher.interfaces import NotFound
from zope.publisher.interfaces.http import IHTTPRequest
from zope.traversing.interfaces import ITraversable, TraversalError


class FilePublisher(grok.View):
    grok.name('file_publish')
    grok.context(IFile)

    def update(self):
        if INamedFile.providedBy(self.context) and self.context.filename:
            self.response.setHeader(
                'Content-Disposition',
                'attachment; filename="%s"' % (
                    self.context.filename.encode('utf-8'))
                )
        self.response.setHeader('Content-Type', self.context.contentType)
        self.response.setHeader('Content-Length', self.context.getSize())

    def render(self):
        return self.context.data


class FileTraverser(grok.MultiAdapter):
    grok.baseclass()
    grok.provides(ITraversable)
    
    def __init__(self, context, request=None):
        self.context = context
        self.request = request
        self.response = request.response

    def get_file(self):
        raise NotImplementedError("Provide your own get_file method")

    def traverse(self, name, ignore=None):
        obj = self.get_file(name)
        if obj is not None:
            if not IFile.providedBy(obj):
                raise TraversalError('%s is not a valid IFile' % name)
            return getMultiAdapter((obj, self.request), name='file_publish')
        raise NotFound(self.context, name, self.request)


class DownloadTraverser(FileTraverser):
    grok.name('download')
    grok.adapts(Interface, IHTTPRequest)

    def get_file(self, name):
        if not checkPermission('zope.View', self.context):
            raise Unauthorized(name)
        return getattr(self.context, name, None)



# -*- coding: utf-8 -*-

import re

_nameFinder = re.compile(r'(.*[\\/:])?(.+)')

def clean_filename(name):
    match = _nameFinder.match(name)
    if match is not None:
        match = match.group(2)
    return unicode(match, 'utf-8')



# -*- coding: utf-8 -*-

from zope.schema import Field
from zope.interface import implements, Interface


class IFileField(Interface):
    """A field storing binary datas.
    """


class IImageField(IFileField):
    """Marker interface for fields storing images.
    """


class FileField(Field):
    """A field handling a file representation
    """
    implements(IFileField)


class ImageField(FileField):
    """A field handling an image file representation
    """
    implements(IImageField)



# -*- coding: utf-8 -*-

from dolmen.file import NamedFile


_marker = object()

class FileProperty(object):
    """Stores the given file data in a NamedFile
    """
    def __init__(self, field, name=None):
        if name is None:
            name = field.__name__

        self.__field = field
        self.__name = name

    def __set__(self, inst, value):
        name = self.__name
        field = self.__field.bind(inst)
        fields = inst.__dict__
        
        if field.readonly and field.has_key(self.__name):
            raise ValueError(self.__name, 'field is readonly')

        if value is not None:
            filename = getattr(value, 'filename', None)
            file = NamedFile(data=value, filename=filename)
        else:
            file = None
            
        fields[name] = file
        inst._p_changed = True

    def __get__(self, inst, klass):
        if inst is None:
            return self

        value = inst.__dict__.get(self.__name, _marker)
        if value is _marker:
            field = self.__field.bind(inst)
            value = getattr(field, 'default', _marker)
            if value is _marker:
                raise AttributeError(self.__name)
        if value:
            value.__parent__ = inst
        return value

    def __getattr__(self, name):
        return getattr(self.__field, name)



# -*- coding: utf-8 -*-

from dolmen.file.utils import clean_filename

from dolmen.file.field import FileField, ImageField
from dolmen.file.field import IFileField, IImageField
from dolmen.file.file import NamedFile, INamedFile
from dolmen.file.access import FilePublisher, FileTraverser, DownloadTraverser
from dolmen.file.property import FileProperty





from os.path import join
from setuptools import setup, find_packages

name = 'dolmen.file'
version = '0.3'
readme = open(join('src', 'dolmen', 'file', "README.txt")).read()
history = open(join('docs', 'HISTORY.txt')).read()

install_requires = [
    'setuptools',
    'grokcore.view',
    'zope.schema',
    'zope.security',
    'zope.interface',
    'zope.component',
    'zope.publisher',
    'zope.traversing',
    'zope.contenttype',
    'zope.app.file',
    ]

tests_require = install_requires + [
    'zope.testing',
    'zope.app.testing',
    'zope.app.zcmlfiles',
    ]

setup(name=name,
      version=version,
      description="A Zope3/Grok File Representation package.",
      long_description = readme + '\n\n' + history,
      keywords='Grok Zope3 file download',
      author='Souheil Chelfouh',
      author_email='trollfot@gmail.com',
      url='',
      license='GPL',
      packages=find_packages('src', exclude=['ez_setup']),
      package_dir={'': 'src'},
      namespace_packages=['dolmen'],
      include_package_data=True,
      zip_safe = False,
      tests_require = tests_require,
      install_requires = install_requires,
      extras_require = {'test': tests_require},
      test_suite="dolmen.file",
      classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Zope3',
        'Intended Audience :: Other Audience',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
          ],
    )
